import { describe, it, expect } from '@jest/globals'

import { viteManifestToDeps } from '../src/manifests.js'

describe('Vite manifest parsing', () => {
  it('works for simple modules', () => {
    const deps = viteManifestToDeps({
      '../io.ox/guidedtours/i18n.de_DE.js': {
        file: 'io.ox/guidedtours/i18n.de_DE.js',
        src: '../io.ox/guidedtours/i18n.de_DE.js',
        isEntry: true,
        meta: {}
      },
      '../io.ox/guidedtours/i18n': {
        file: 'io.ox/guidedtours/i18n.3de05d46.js',
        src: '../io.ox/guidedtours/i18n',
        isEntry: true,
        imports: [
          '_preload-helper-a7bbbf37.js'
        ],
        meta: {
          gettext: {
            dictionary: true
          },
          manifests: [
            {
              namespace: 'i18n'
            }
          ]
        }
      },
      'io.ox/guidedtours/intro.js': {
        file: 'io.ox/guidedtours/intro.e84819a5.js',
        src: 'io.ox/guidedtours/intro.js',
        isEntry: true,
        isDynamicEntry: true,
        imports: [
          '../io.ox/guidedtours/i18n',
          '_preload-helper-a7bbbf37.js'
        ],
        meta: {}
      },
      'io.ox/guidedtours/main.js': {
        file: 'io.ox/guidedtours/main.07676e21.js',
        src: 'io.ox/guidedtours/main.js',
        isEntry: true,
        imports: [
          '_preload-helper-a7bbbf37.js',
          '../io.ox/guidedtours/i18n'
        ],
        dynamicImports: [
          'io.ox/guidedtours/intro.js',
          'io.ox/guidedtours/multifactor.js'
        ],
        meta: {
          manifests: [
            {
              namespace: 'settings'
            },
            {
              namespace: 'io.ox/core/main',
              title: 'Guided tours',
              company: 'Open-Xchange',
              icon: '/images/icon.png',
              category: 'Dev',
              settings: false,
              index: 100,
              package: 'open-xchange-guidedtours'
            }
          ]
        }
      },
      'io.ox/guidedtours/multifactor.js': {
        file: 'io.ox/guidedtours/multifactor.22d3e17d.js',
        src: 'io.ox/guidedtours/multifactor.js',
        isEntry: true,
        isDynamicEntry: true,
        imports: [
          '_preload-helper-a7bbbf37.js',
          '../io.ox/guidedtours/i18n',
          'io.ox/guidedtours/main.js'
        ],
        css: [
          'io.ox/guidedtours/assets/multifactor.91962241.css'
        ],
        meta: {}
      },
      'io.ox/guidedtours/utils.js': {
        file: 'io.ox/guidedtours/utils.91ad511f.js',
        src: 'io.ox/guidedtours/utils.js',
        isEntry: true,
        imports: [
          '_preload-helper-a7bbbf37.js'
        ],
        meta: {}
      },
      '_preload-helper-a7bbbf37.js': {
        file: 'io.ox/guidedtours/preload-helper-a7bbbf37.js'
      }
    })
    expect(typeof deps).toBe('object')
    expect(Object.keys(deps).length).toBe(8)
    expect(deps['io.ox/guidedtours/main.07676e21.js']).toEqual(['io.ox/guidedtours/preload-helper-a7bbbf37.js', 'io.ox/guidedtours/i18n.3de05d46.js'])
    expect(deps['io.ox/guidedtours/multifactor.22d3e17d.js']).toEqual(['io.ox/guidedtours/preload-helper-a7bbbf37.js', 'io.ox/guidedtours/i18n.3de05d46.js', 'io.ox/guidedtours/main.07676e21.js', 'io.ox/guidedtours/assets/multifactor.91962241.css'])
  })

  it('exports assets as entrypoints without dependencies', async () => {
    const deps = viteManifestToDeps({
      'themes/icons/alarm.svg': {
        file: 'assets/alarm.6d2fbb40.js',
        src: 'themes/icons/alarm.svg',
        isDynamicEntry: true,
        assets: [
          'assets/alarm.310541a0.svg'
        ],
        meta: {}
      }
    })
    expect(Object.keys(deps).length).toBe(2)
    expect(deps['assets/alarm.6d2fbb40.js']).toEqual(['assets/alarm.310541a0.svg'])
    expect(deps['assets/alarm.310541a0.svg']).toEqual([])
  })

  it('exports css as entrypoints without dependencies', async () => {
    const deps = viteManifestToDeps({
      'main.js': {
        file: 'main.js',
        src: 'main.js',
        isEntry: true,
        css: ['assets/main.3b761440.css'],
        meta: {}
      }
    })
    expect(deps['main.js']).toEqual(['assets/main.3b761440.css'])
    expect(deps['assets/main.3b761440.css']).toEqual([])
  })

  it('separately exports HTML entrypoints', async () => {
    const deps = viteManifestToDeps({
      'index.html': {
        file: 'index.html.js',
        src: 'index.html',
        isEntry: true,
        imports: ['main.js', '_preload-helper.a295b1c6.js', '_vendor.ae457d06.js'],
        meta: {}
      },
      '_preload-helper.a295b1c6.js': { file: 'assets/preload-helper.a295b1c6.js' },
      '_vendor.ae457d06.js': {
        file: 'assets/vendor.ae457d06.js',
        isDynamicEntry: true,
        dynamicImports: ['io.ox/core/a11y.js']
      },
      'io.ox/core/a11y.js': {
        file: 'io.ox/core/a11y.js',
        src: 'io.ox/core/a11y.js',
        isEntry: true,
        isDynamicEntry: true,
        imports: ['_vendor.ae457d06.js'],
        meta: {}
      },
      'main.js': {
        file: 'main.js',
        src: 'main.js',
        isEntry: true,
        imports: ['_preload-helper.a295b1c6.js', '_vendor.ae457d06.js', 'io.ox/core/a11y.js'],
        css: ['assets/main.3b761440.css'],
        meta: {}
      }
    })

    expect(Object.keys(deps).length).toBe(7)
    expect(deps['index.html']).toEqual(['index.html.js'])
  })
})
